/* */

signal on break_c

/*LIBRARIES*/
if ~show("L","rmh.library") then
    if ~addlib("rmh.library",0,-30) then do
		say "no rmh.library"
		exit
	end
if ~show("L","rxsocket.library") then
    if ~addlib("rxsocket.library",0,-30) then do
		say "no rxsocket.library"
		exit
	end
if ~show("L","rxmiamibpf.library") then
    if ~addlib("rxmiamibpf.library",0,-30) then do
		say "no rxmiamibpf.library"
		exit
	end

/*ARGUMENTS*/
prg=ProgramName("NOEXT")
if ~RMH_ReadArgs("INTERFACE,FILTERS/K/M") then do
	call PrintFault(IoErr(),prg)
	exit
end

if parm.0.flag then do
	interface=parm.0.value
	res=IsOnLine(parm.0.value)
	if res<0 then call err "interface '"interface"' doesn't exist"
	if res==0 then call err "interface '"interface"' is not up"
end
else interface=""

call initDB

ctrlc=2**12
bpfa.device    = interface
bpfa.BuffLen   = 256
bpfa.Immediate = 1
bpfa.AbortMask = ctrlc
if parm.1.flag then bpfa.Filter=parm.1.value.0
bpf=MiamiBPFOpen("BPFA")
if bpf<0 then call err BPFERR

call MiamiBPFIoctlGet(bpf,"D","DEVICE")
say "Listening on" d

cs = MiamiBPFSignalmask(bpf)
call MiamiBPFSetAbortMask(bpf,ctrlc)
sig=or(cs,ctrlC)


t.secs=GMTOffset()*60

do numMult=1 to parm.1.mult-1
	f.numMult=CreateFilter(bpf,parm.1.value.numMult)
	if f.i<0 then call err BPFERR
end

do while 1
	call MiamiBPFIoctlGet(bpf,"BS","FIONREAD")
	if bs<=0 then res=Wait(sig)
	else if CheckSignal(ctrlc) then res = ctrlc
		 else res=0
	if and(res,ctrlc)~=0 then signal break_c
	call MiamiBPFIoctlGet(bpf,"BS","FIONREAD")
	if bs>0 then do
		l=MiamiBPFRead(bpf,"PKT",1)
		if l>0 then call parsePkt(pkt)
	end
end

rresolve: procedure
parse arg ip
	if ~GethostByAddr("H",ip) then return ip
	return h.hostname

resolveport: procedure
parse arg port,proto
	if ~GetServByPort("S",port,proto) then return port
	return s.servname

parsepkt:
parse arg pkt
	call SubTime("PKT","T")
	call TV2DS("PKT")
	date=FormatDate("PKT","%I:%M:%S")||"."||pkt.micro

	call ReadIP(substr(pkt,37,100),"IPH")
	from=rresolve(iph.src)
	to=rresolve(iph.dst)
	packetlen=iph.len

	select
		when iph.p==1 then do
			call readICMP(substr(pkt,57,100),"ICMPH")
			t=icmph.type
			c=icmph.code
			if value("DB.ICMP."t"."c)~="DB.ICMP."t"."c then c=db.icmp.t.c
			if value("DB.ICMP.TYPE."t)~="DB.ICMP.TYPE."t then t=db.icmp.type.t
			hh = date "len:"packetlen from ">" to": icmp type:"t "code:"c
		end

		when iph.p==6 then do
			call readTCP(substr(pkt,57,100),"TCPH")
			if tcph.flags==0 then fl="."
			else do
				fl=""
				if and(tcph.flags,1)~=0 then fl=fl"F"
				if and(tcph.flags,2)~=0 then fl=fl"S"
				if and(tcph.flags,4)~=0 then fl=fl"R"
				if and(tcph.flags,8)~=0 then fl=fl"P"
				if and(tcph.flags,16)~=0 then fl=fl"A"
				if and(tcph.flags,32)~=0 then fl=fl"U"
			end
			hh = date "len:" packetlen from"."resolveport(tcph.sport,"tcp") ">" to"."resolveport(tcph.dport,"tcp") "tcp" fl
		end

		when iph.p==17 then do
			call readUDP(substr(pkt,57,100),"UDPH")
			hh = date "len:"packetlen from"."resolveport(udph.sport,"udp") ">" to"."resolveport(udph.dport,"udp") "udp"
		end

		otherwise do
			hh = date "len:"packetlen "from ">" to: protocol" iph.p
		end
	end
	fs=""
	do i=1 to numMult-1
		if MiamiBPFFilter(f.i,pkt) then fs=i fs
	end
	say fs hh
	return

initDB: procedure expose db.
	db.icmp.type.0="echo reply"
	db.icmp.type.3="dest unreacheble"
	db.icmp.type.4="packet lost"
	db.icmp.type.5="shorter route"
	db.icmp.type.8="echo request"
	db.icmp.type.9="router advertisement"
	db.icmp.type.10="router solicitation"
	db.icmp.type.11="time exceeded"
	db.icmp.type.12="ip header bad"
	db.icmp.type.13="timestamp request"
	db.icmp.type.14="timestamp reply"
	db.icmp.type.15="information request"
	db.icmp.type.16="address mark request"
	db.icmp.type.17="address mark reply"
	return

break_c:
	call MiamiBPFIoctlGet(bpf,"S","STAT")
	say s.recv "packets received by filter"
	say s.drop "packets dropped by kernel"
	exit

err: procedure expose prg
parse arg msg
	say prg":" msg
	exit
